# $Id: sound.tcl,v 1.8 2003/06/04 07:22:46 aleksey Exp $

namespace eval sound {
    custom::defgroup Sound [::msgcat::mc "Sound options."] -group Tkabber

    variable options
    variable groupchat_may_notify
    # Sound is on by default
    set options(sound) 1
    custom::defvar options(mute) 0 \
	    [::msgcat::mc "Mute sound notifying."] -group Sound -type boolean
    variable mute 0
    custom::defvar options(mute_groupchat_delayed) 1 \
	    [::msgcat::mc "Mute sound when displaying delayed groupchat messages."] -group Sound -type boolean
    custom::defvar options(mute_chat_delayed) 0 \
	    [::msgcat::mc "Mute sound when displaying delayed personal chat messages."] -group Sound -type boolean
    # One could use external play program instead of Snack (default for unix)
    custom::defvar options(external_play_program) \
	    [expr {($::tcl_platform(platform) == "unix") ? "esdplay" : ""}] \
	    [::msgcat::mc "External program, which is to be executed to play sound. If empty, Snack library is required to play sound."] \
	    -group Sound -type string
    # Params for external play program
    custom::defvar options(external_play_program_options) "" \
	[::msgcat::mc "Options for external play program"] \
	-group Sound -type string
    # Sound theme
    custom::defvar options(theme) "default" \
	    [::msgcat::mc "Sound theme. If it starts with \"/\" or \"~\" then it is considered as theme directory. Otherwise theme is loaded from \"sounds\" directory of Tkabber tree."] -group Sound -type string
    variable play_id ""
    variable play_priority 0
    # Do not allow play sound very often
    custom::defvar options(delay) 200 \
	    [::msgcat::mc "Time interval before playing next sound (in milliseconds)."] -group Sound \
	    -type integer

    # MUC "End of History" marker
    set options(end_of_history) "-"
}

proc sound::sound_setup {} {
    variable options
    variable groupchat_may_notify
    variable sounds

    if {!$options(sound)} return

    set names [list groupchat_server_message groupchat_my_message \
		    groupchat_their_message chat_my_message \
		    chat_their_message connected \
		    presence_available presence_unavailable \
		    groupchat_their_message_to_me]

    foreach name $names {
	if {[regexp {^[/~]} $options(theme)]} {
	    set fname [file nativename [file join $options(theme) $name.wav]]
	} else {
	    set fname [fullpath sounds $options(theme) $name.wav]
	}
	if {[file exist $fname]} {
	    set sounds($name) $fname
	} else {
	    set sounds($name) ""
	}
    }

    if {$options(external_play_program) == ""} {
	if {[catch { package require snack 2.0 }]} {
	    debugmsg tkabber \
		"unable to load the Snack package, so no sound support!\n\nThe Snack package is available at http://www.speech.kth.se/snack/index.html"
	    set options(sound) 0
	    return
	}

	foreach name $names {
	    if {$sounds($name) != ""} {
		snack::sound $sounds($name) -file $sounds($name)
	    }
	}
    }
}

proc sound::play {name {priority 0}} {
    global userstatus
    variable options
    variable mute
    variable play_id
    variable play_priority

    if {$name == ""} return
    if {$options(mute) || $mute} return

    if {$play_id != ""} {
	if {$priority >= $play_priority} {
	    return
	} else {
	    after cancel $play_id
	}
    }
    if {$options(delay) > 0} {
	set play_id [after $options(delay) [list set [namespace current]::play_id {}]]
    }
    set play_priority $priority

    if {$options(external_play_program) == ""} {
	catch { $name play -block 0 }
    } else {
	catch { eval "exec $options(external_play_program) $options(external_play_program_options) [list $name] &" }
    }
}

proc sound::chat_message_notify_setup {jid type} {
    variable options
    variable groupchat_may_notify

    if {$type == "groupchat"} {
	# Skip sound notifying on first messages until the first server message coming
	# Reasonable?
	# Now another mechanism of sound muting is used
	set groupchat_may_notify($jid) 1
    }
}

proc sound::chat_message_notify {jid from type body extras} {
    variable options
    variable sounds
    variable groupchat_may_notify

#    if {[cequal $type groupchat] && !$groupchat_may_notify($jid) && [string equal $jid $from]} {
#	if {!$::muc::muc_compatible($jid) || [string equal $body $options(end_of_history)]} {
#	    set groupchat_may_notify($jid) 1
#	}
#    }

    if {!$options(sound)} return

    set delayed 0
    foreach xelem $extras {
	jlib::wrapper:splitxml $xelem tag vars isempty chdata children

	if {[cequal [jlib::wrapper:getattr $vars xmlns] jabber:x:delay]} {
	    set delayed 1
	}
    }

    switch -- $type {
	groupchat {
	    if {$delayed && $options(mute_groupchat_delayed)} {
		return
	    }
	    if {$groupchat_may_notify($jid)} {
		if {[string equal $jid $from]} {
		    play $sounds(groupchat_server_message)
		} elseif {[chat::is_our_jid $jid $from]} {
		    play $sounds(groupchat_my_message)
		} else {
		    set myjid [chat::our_jid $jid]
		    set mynick [chat::get_nick $myjid $type]
		    if {[crange $body 0 [expr [clength $mynick] + 1]] == "${mynick}: "} {
			play $sounds(groupchat_their_message_to_me) -1
		    } else {
			play $sounds(groupchat_their_message)
		    }
		}
	    }
	}
	chat {
	    if {$delayed && $options(mute_chat_delayed)} {
		return
	    }
	    if {[chat::is_our_jid $jid $from]} {
		play $sounds(chat_my_message)
	    } else {
		play $sounds(chat_their_message) -1
	    }
	}
    }
}

proc sound::presence_notify {jid status} {
    variable options
    variable sounds

    if {!$options(sound)} return

    if {$status == "unavailable"} {
	play $sounds(presence_unavailable)
    } else {
	play $sounds(presence_available)
    }
}

proc sound::mute_setup {status} {
    variable mute

    switch -- $status {
	xa -
	dnd {
	    set mute 1
	}
	default {
	    set mute 0
	}
    }
}

proc sound::connected_notify {} {
    variable options
    variable sounds

    if {!$options(sound)} return

    play $sounds(connected) 1
}

proc sound::clean_chat_message_notify {} {
    variable groupchat_may_notify

    foreach n [array names groupchat_may_notify] {
	set groupchat_may_notify($n) 0
    }
}

hook::add on_change_user_presence_hook [namespace current]::sound::presence_notify 100
hook::add change_our_presence_post_hook [namespace current]::sound::mute_setup 100
hook::add connected_hook [namespace current]::sound::connected_notify 100
hook::add disconnected_hook [namespace current]::sound::clean_chat_message_notify 100
hook::add postload_hook [namespace current]::sound::sound_setup 100
hook::add open_chat_post_hook [namespace current]::sound::chat_message_notify_setup 60
hook::add draw_message_hook [namespace current]::sound::chat_message_notify 19

